<?php

namespace Drupal\tmgmt_wordsonline\Form;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Database;
use Drupal\Core\Url;
use Drupal\Core\Routing;
use Drupal\Component\Render\FormattableMarkup;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Archiver\Zip;
use Drupal\Core\Archiver\ArchiverException;
use Drupal\file\Entity\File;
class UpdateForm extends FormBase{
    const FILE_EXISTS_REPLACE = 1;
    /**
     * {@inheritdoc}
     */
    public function getFormId() {
        return 'wol_update_form';
    }
      /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state, $files = []) {
      $form['new_file'] = [
            '#type' => 'managed_file',
            '#title' => t('Update file'),
            '#description' => t('Allowed extensions: zip'),
            '#required' => TRUE,
            '#upload_location' => 'public://wordsonline_upload',
            '#upload_validators' => [
              'file_validate_extensions' => ['zip'],
            ],
          ];
      $form['#attributes']['enctype'] = 'multipart/form-data';
      $form['actions']['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Update'),
            '#name' => 'update' ,
        ];
      return $form;

    }
  /**
   * remove old version zip files
   *
   */
  public function remove($fileName){
    try{

      $absolute_path = \Drupal::service('file_system')->realpath('public://wordsonline_upload');

      $files = glob("{$absolute_path}/*"); // get all file names
      foreach($files as $file){ // iterate files
    
        if(is_file($file)) {
          $public_path = str_replace($absolute_path,'public://wordsonline_upload',$file);
          if($public_path ==$fileName)continue;
          unlink($file); // delete file
          $query = \Drupal::database()->delete('file_managed');
          $query->condition('uri', $public_path);
           $query->execute();
        }
      }

    }catch(Exception $e) {
      \Drupal::logger('tmgmt_wordsonline_configs')->error($e->getMessage());
    }
  }

   /**
   * update version by replace files
   *
   */
   public function update(array $form, FormStateInterface $form_state) {
    
    try{
        $field = $form_state->getUserInput();    
        $file = $field['new_file'];
        $fids = $file['fids'];
        $f_ret = \Drupal::database()->query("SELECT  * FROM file_managed where fid =:fid LIMIT 1",[':fid'=>$fids])->fetchAssoc(); 
        if($f_ret!=null){
           $file_path = $f_ret['uri'];
           $this->remove($file_path);
           $module_path =\Drupal::service('file_system')->realpath(\Drupal::service('module_handler')->getModule('tmgmt_wordsonline')->getPath());
           $zip = \Drupal::service('plugin.manager.archiver')->getInstance(['filepath' => $file_path]);
           try {
                $zip->extract($module_path);            
                $files =$zip->listContents();              
              }
            catch (ArchiverException $exception) {
                watchdog_exception('tmgmt_wordsonline', $exception);
                \Drupal::messenger()->addMessage("Update error");
            }
        }
        $url = new Url('tmgmt_wordsonline.wol_update', [], []);
        $url->setAbsolute();
        $response = new RedirectResponse($url->toString());
        $response->send();
        \Drupal::messenger()->addMessage("Update successful");
         
    }catch (Exception $e) 
    {
      \Drupal::logger('tmgmt_wordsonline_configs')->error($e->getMessage());
      $url = new Url('tmgmt_wordsonline.wol_update', [], []);
      $url->setAbsolute();
      $response = new RedirectResponse($url->toString());
      $response->send();
      \Drupal::messenger()->addMessage("Update error");
      return;
    }

  }

  /**
  * {@inheritdoc}
  */
  public function validateForm(array & $form, FormStateInterface $form_state) {
     
  }
   
  /**
  * {@inheritdoc}
  */
  public function submitForm(array & $form, FormStateInterface $form_state) {
      $this->update($form,$form_state); 
  }
}