<?php

namespace Drupal\tmgmt_wordsonline;

use Drupal\Component\Utility\NestedArray;
use Drupal\tmgmt\TranslatorPluginUiBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\tmgmt\JobInterface;
use Drupal\Core\Database;
use Drupal\tmgmt_wordsonline\WordsOnlineMessage;
use Drupal\tmgmt_wordsonline\Entity\WOLLangPair;
use Drupal\tmgmt_wordsonline\Entity\WOLServiceLevel;
/**
 * WordsOnline translator UI.
 */
class WordsOnlineTranslatorUi extends TranslatorPluginUiBase {
  public $services;
  public $default_service_level;
  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $translator = $form_state->getFormObject()->getEntity();
    $database = \Drupal::database();
    $results =  $database->select('tmgmt_wordsonline_configs', 'wol')
                         ->extend('\Drupal\Core\Database\Query\PagerSelectExtender')
                         ->fields('wol')
                         ->limit(1)
                         ->execute()
                         ->fetchAssoc();
    $username =null;
    $password = null;
    $project_key = null;
    if($results != null){
         $username =$results["username"];
         $password =$results["password"];
         $project_key =$results["project_key"];
    }
    $form['username'] = [
        '#type' => 'textfield',
        '#title' => t('Username'),
        '#value' =>  $username,
        '#description' => t('Please enter the username.'),
        '#required' => TRUE,
        '#placeholder' => 'Please enter the username.',
        '#prefix' => '<div class="wol-text-field">',
        '#suffix' => '</div>',
      ];
     
    
    $form['password'] = [
        '#type' => 'password',
        '#title' => t('Password'),
        '#value' => $password,
        '#description' => t('Please enter the password.'),
        '#required' => TRUE,
        '#placeholder' => 'Please enter the password.',
        '#attributes' => array('value' => $password),
        '#prefix' => '<div class="wol-text-field">',
        '#suffix' => '</div>',
      ];
    $form['project_key'] = [
        '#type' => 'textfield',
        '#title' => t('Project ID'),
        '#value' => $project_key,
        '#description' => t('Please enter the Project ID.'),
        '#required' => TRUE,
        '#placeholder' => 'Please enter the Project ID.',
        '#prefix' => '<div class="wol-text-field">',
        '#suffix' => '</div>',
      ];
    $form['grant_type'] = [
        '#type' => 'hidden',
        '#default_value' =>  'password',
        '#value'=>'password',
      ];
    $form['scope'] = [
        '#type' => 'hidden',
        '#default_value' =>  'enterprise',
        '#value'=>'enterprise',
      ];
    $form += parent::addConnectButton();
    $form['#attached']['library'][] = 'tmgmt_wordsonline/wordsonline_form'; 

    return $form;
  }
  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);
    if ($form_state->hasAnyErrors()) {
      return;
    }
    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $translator = $form_state->getFormObject()->getEntity();
    /** @var \Drupal\tmgmt_wordsonline\Plugin\tmgmt\Translator\WordsOnlineTranslator $plugin */
    $plugin = $translator->getPlugin();
    $plugin->setTranslator($translator);
   
    try{
     
      $field = $form_state->getUserInput();         
      $fields["username"] = $field['settings']['username'];
      $fields["password"] = $field['settings']['password'];
      $fields["scope"] = $field['settings']['scope'];
      $fields["grant_type"] = $field['settings']['grant_type'];

      $result = getStatusCode( 'POST',$fields);
      if ($result == 401) {
        $form_state->setErrorByName('settings][username', t(WordsOnlineMessage::AUTHEN_ERROR));
      }
      
      if($result== 200){
        try{         
          $conn = \Drupal\Core\Database\Database::getConnection();
          $conn->delete('tmgmt_wordsonline_configs')->execute();
          $f= [
            'username',
            'password',
            'scope',
            'grant_type',
            'api_url',
            'config_id',
            'project_key'
          ];
          $values= [
            'username' => $fields["username"],
            'password' => $fields["password"],
            'scope' => $fields["scope"] ,
            'grant_type' => $fields["grant_type"],
            'api_url' =>  WordsOnlineConst::API_URL,
            'config_id' => 1 ,
            'project_key'=> $field['settings']['project_key']
          ];
          $conn->insert('tmgmt_wordsonline_configs')
               ->fields($f)->values($values)->execute();
          \Drupal::messenger()->addMessage($this->t(WordsOnlineMessage::AUTHEN_SAVED));
           
        } catch(Exception $ex){
          \Drupal::logger('tmgmt_wordsonline_configs')->error($ex->getMessage());
        }
      }
    }catch (Exception $e) 
    {
      \Drupal::logger('tmgmt_wordsonline_configs')->error($e->getMessage());
    }
   
  }
  /**
   * {@inheritdoc}
   */
  public function checkoutSettingsForm(array $form, FormStateInterface $form_state, JobInterface $job) {
    /** @var \Drupal\tmgmt_wordsonline\Plugin\tmgmt\Translator\WordsOnlineTranslator $translator_plugin */
    $translator = $job->getTranslator();
    $translator_plugin =  $translator->getPlugin();
    $translator_plugin->setTranslator($translator);
    $triggering_element = $form_state->getTriggeringElement();
    $translator_settings = NestedArray::mergeDeep($job->getTranslatorPlugin()->defaultSettings(), $translator->getSettings());
    $projects = array();
    $results = \Drupal::database()->query(WordsOnlineConst::SELECT_CONFIG)->fetchAssoc(); 
    $contentsarray = array();
    $project_key =$job->getSetting('project_key');
    $timezone = date_default_timezone_get();
    if($results != null){
      $auth["username"] = $results["username"];
      $auth["password"] = $results["password"];
      $auth["scope"] = $results["scope"];
      $auth["grant_type"] =$results["grant_type"];
      $response = getToken('POST', $auth);
      $response = json_decode($response, TRUE);
      $token =$response['access_token'];
    if($project_key == null){
          $project_key = $results['project_key'];
    }
    if($token == null || $token == ''){
      \Drupal::messenger()->addError(t('WordsOnline provider settings incorrect. Please contact <a href="mailto:helpdesk@wordsonline.com"> helpdesk@wordsonline.com</a> to obtain WordsOnline provider settings.'));
      return;
    }
    $timezone =timezone_name_from_abbr($response['timezone']);
    $projects = $this->getProject('POST',$project_key,$token);
    if($projects){
      foreach ($projects['contentTypes'] as $row){
        $contentsarray[$row] =$row;
      }
      $this->services = $projects["serviceLevels"];
    }
    }else{
      \Drupal::messenger()->addError(t('Cannot find WordsOnline provider settings. Please contact <a href="mailto:helpdesk@wordsonline.com"> helpdesk@wordsonline.com</a> to obtain WordsOnline provider settings.'));
    }

    $form['project_key'] = [
      '#type' => 'hidden',
      '#required' => TRUE,
      '#default_value' => $project_key,
      '#value'=>$project_key,
    ];

    $form['content_type'] = array(
      '#title' => t('Content Type'),
      '#type' => 'select',
      '#required' => TRUE,
      "#empty_option"=>t('- Select -'),
      '#options' => $contentsarray,
      '#default_value' => $job->getSetting('content_type'),
      '#prefix' => '<div class="wol-text-field">',
      '#suffix' => '</div>',
      "#attributes" =>[
          'service-level-data' =>json_encode($this->services),
          'onchange' => 'onContentTypeChange(this)'
        ]

    );

    $this-> default_service_level = $job->getSetting('serice_level');
    $servicesarray = array();
    $servicearray["Basic"] = "Basic";
    $servicearray[ "Standard"] = "Standard";
    $servicearray[ "Enterprise"] = "Enterprise";
    $servicearray["Premium"] = "Premium";
    $form['service_level'] = array(
      '#title' => t('Service Level'),
      '#type' => 'select',
      '#required' => TRUE,
      '#default_value' =>  $this-> default_service_level,
      '#options' => $servicearray,
      '#prefix' => '<div class="wol-text-field" id="service_level_wrapper">',
      '#suffix' => '</div>'
    );
    $form['due_date'] =  array(
      '#title' => t('Due Date ('.$timezone.')'),
      '#type' => 'date',
      '#default_value' => $job->getSetting('due_date'),
      '#format' => 'YYY-MM-DD h:mm',
      '#required' => TRUE,
      '#attributes' => [
          'min' =>  (new \DateTime("tomorrow", new \DateTimeZone($timezone)))->format('Y-m-d'),//date('Y-m-d'),
          'onchange' => "onDateChage(this)"
        ],
      '#prefix' => '<div class="wol-text-field wol-des">',
      '#suffix' => '</div>',
      );
      $request_name =$job->getSetting('request_name');
      if(!$request_name){
        $request_name = getRequestName();
      }
      $form['request_name'] = [
        '#type' => 'textfield',
        '#title' => t('Request Name'),
        '#default_value' =>  $request_name,
        '#required' => TRUE,
        '#placeholder' => 'Please enter the request name.',
        '#prefix' => '<div class="wol-text-field">',
        '#suffix' => '</div>',
        ];
      $form['auto_approve_quote'] =  array(
        '#title' => t('Auto Approve Quote'),
        '#description' => t('Quote of this order will be approved automatically.'),
        '#type' => 'checkbox', 
        '#default_value' =>$job->getSetting('auto_approve_quote'),
        '#prefix' => '<div class="wol-checkbox-field">',
        '#suffix' => '</div>',
        );
      $is_auto=$translator->isAutoAccept();
         
      if($is_auto == true ||$is_auto == 1){
        $form['auto_import'] =  array(
          '#title' => t('Auto import upon receiving delivery files'),
          '#description' => t('Auto import the translation back to Drupal when the files delivered.'),
          '#type' => 'checkbox', 
          '#default_value' => true,
          '#prefix' => '<div class="wol-checkbox-field d-hidden" >',
          '#suffix' => '</div>',
          );
      }else{
        $form['auto_import'] =  array(
          '#title' => t('Auto import upon receiving delivery files'),
          '#description' => t('Auto import the translation back to Drupal when the files delivered.'),
          '#type' => 'checkbox', 
          '#default_value' => $job->getSetting('auto_import'),
          '#prefix' => '<div class="wol-checkbox-field">',
          '#suffix' => '</div>',
          );
      }
      $form['#attached']['library'][] = 'tmgmt_wordsonline/wordsonline_form'; 
      return parent::checkoutSettingsForm($form, $form_state, $job);
  }
 /**
  * get list wordsonline project
  *
  */
  function getProject( $method = 'POST',$project_key,$token = NULL) {
    $options = [];
    $projects =array();
    $url = WordsOnlineConst::API_URL  . WordsOnlineConst::GET_PROJECT_URL.$project_key;
    $client = \Drupal::httpClient();
  
    try {
      $options['headers'] = [
        'Authorization' => 'Bearer '.$token,
        'Referer'=>'ClientAPI',
        'Content-Type' => 'application/json',
      ];
      $options['timeout']= 3600;
      $response = $client->request($method, $url, $options);
   
    }
    catch (RequestException $e) {
      if (!$e->hasResponse()) {      
        throw new TMGMTException(WordsOnlineMessage::UNABLE_TO_CONNECT_SERVICES, ['@error' => $e->getMessage()], $e->getCode());
      }
      $response = $e->getResponse(); 
      throw new TMGMTException(WordsOnlineMessage::UNABLE_TO_CONNECT_SERVICES, ['@error' => $response->getReasonPhrase()], $response->getStatusCode());
    }
    $received_data = $response->getBody()->getContents();
    if ($response->getStatusCode() != 200) {
      throw new TMGMTException(WordsOnlineMessage::UNABLE_TO_CONNECT_SERVICES_WITH_URL,
        ['@error' => $response->getStatusCode(), '@url' => $url]);
    }

    $orderResponse = json_decode($received_data, TRUE);
    if($orderResponse["status"] ==1){
      foreach ($orderResponse["result"]["languagePairs"] as $row){
        $projects["languages"][] =new WOLLangPair($row['sourceLanguage'],$row['targetLanguage']);
      }
      foreach ($orderResponse["result"]["contentTypes"] as $row){
        $projects["contentTypes"][] =$row;
      }
      foreach ($orderResponse["result"]["serviceLevels"] as $row){
        $projects["serviceLevels"][] =new WOLServiceLevel($row['name'],$row['contentType']);
      }
    }

    return $projects;
  }
   /**
   * Submit callback to pull translations from WordsOnlineTranslator.
   */
  public function submitPullTranslations(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\tmgmt\Entity\Job $job */
    $job = $form_state->getFormObject()->getEntity();
    /** @var \Drupal\tmgmt_wordsonline\Plugin\tmgmt\Translator\WordsOnlineTranslator $translator_plugin */
    $translator_plugin = $job->getTranslator()->getPlugin();
    $translator_plugin->fetchTranslations($job);
    tmgmt_write_request_messages($job);
  }
}